#ifndef OCARINA_PLAYLIST_H
#define OCARINA_PLAYLIST_H

#include <ocarina/ocarina.h>
#include <libsaria/playlist.h>
#include <libsaria/renderer.h>

namespace ocarina
{

	class Playlist : public libsaria::PlaylistRenderer {
		private:
			bool visible;
			GtkWidget *name;
			GtkWidget *number;
			GtkWidget *box;
			GtkWidget *entry;
			GtkWidget *treeview;
			GtkWidget *disable_button;
			GtkListStore *liststore;
			GtkTreeModel *filter;
			GtkTreeSelection *treesel;

			void set_label_text();
			void set_length_label_text();
			void setup_treeview();
			GtkWidget *make_disable_button();

		public:
			Playlist(unsigned int);
			Playlist(libsaria::Playlist *);
			~Playlist();

			void on_show();
			void on_hide();
			void switch_to();
			void right_click(guint, guint64);
			void select_none();
			void list_selected_tracks(list<libsaria::Track *> &);
			void list_selected_indices(list<unsigned int> &);

			void set_playlist(libsaria::Playlist *);
			void renumbered(int);
			void refilter();
			void track_updated(libsaria::Track *);
			void prepare_for_removal();
			void on_disable(bool);

			void goto_index(unsigned int);

			void modify_prepare();
			void insert(libsaria::Track *, unsigned int);
			void modify_done();

			void remove_index(unsigned int);
	};

	namespace playlist
	{

		void switch_to_library();
		void switch_to_recent();
		void switch_to_banned();
		void switch_to_plist(unsigned int);
		void init();

	};

}; /* Namespace: ocarina */

#endif /* OCARINA_PLAYLIST_H */
