// Copyright (c) 2011 Bryan Schumaker.
#include <libsaria/idle.h>
#include <libsaria/print.h>

#include <list>
using namespace std;

static bool enabled = false;
static list<IdleTask *> idle_queue;
static float queued = 0.0;
static float serviced = 0.0;

static void do_task(IdleTask *task)
{
	task->run_task();
	delete task;
}

namespace libsaria
{

	float idle::progress()
	{
		if (queued == 0)
			return 1.0;
		return (serviced / queued);
	}

	int idle::size()
	{
		return idle_queue.size();
	}

	void idle::run_task()
	{
		IdleTask *task;
		if (size() == 0)
			return;
		task = idle_queue.front();
		idle_queue.pop_front();
		do_task(task);
		if (size() == 0) {
			queued = 0.0;
			serviced = 0.0;
		} else
			serviced += 1.0;
	}

	/*
	 * IDLE_TASK_QUEUED will only be triggered when the queue
	 * size is increased from 0 to 1.  This will tell the UI
	 * that idle tasks have been queued.
	 */
	void idle::queue_task(IdleTask *task, bool front)
	{
		if (enabled == false) {
			do_task(task);
			return;
		}

		if (front == true)
			idle_queue.push_front(task);
		else
			idle_queue.push_back(task);
		queued += 1.0;
	}

	void idle::cancel_all(void *data)
	{
		list<IdleTask *>::iterator it;
		for (it = idle_queue.begin(); it != idle_queue.end(); it++) {
			if ((*it)->should_cancel(data)) {
				it = idle_queue.erase(it);
				it--;
			}
		}
	}

	void idle::enable()
	{
		enabled = true;
	}

};
