// Copyright (c) 2012 Bryan Schumaker.
#include <libsaria/playlist.h>
#include <libsaria/renderer.h>
#include <libsaria/track.h>
#include <libsaria/print.h>

static bool compare_tracks(libsaria::Track *one, libsaria::Track *two)
{
	/* I want to compare tracks and not pointers */
	return *one < *two;
}

namespace libsaria
{

	void Playlist::add_track(Track *track, unsigned int ins_index)
	{
		length += track->get_length();
		index.add_track(track);
		track->add_playlist(this);
		RENDER( insert(track, ins_index) );
		data_state = DIRTY;
	}

	unsigned int Playlist::remove_track_it(list<Track *>::iterator &it,
					       unsigned int rm_index)
	{
		bool removing_cur = (it == cur);
		length -= (*it)->get_length();
		index.remove_track(*it);
		(*it)->rm_playlist(this);
		it = plist.erase(it);
		RENDER( remove_index(rm_index) );
		if (rm_index != 0) {
			rm_index--;
			it--;
		} else
			it = plist.begin();
		if (removing_cur)
			cur = it;
		data_state = DIRTY;
		return rm_index;
	}

	void Playlist::remove_tracks(list<Track *> &tracks)
	{
		list<Track *>::iterator p_it, t_it;
		unsigned int index = 0;

		RENDER( modify_prepare() );

		for (p_it = plist.begin(); p_it != plist.end(); p_it++) {

			for (t_it = tracks.begin(); t_it != tracks.end(); t_it++) {
				if (*p_it == *t_it) {
					remove_track_it(p_it, index);
					break;
				}
			}

			if (t_it == tracks.end())
				index++;
		}

		schedule_save();
		RENDER( modify_done() );
	}

	void Playlist::remove_indices(list<unsigned int> &indices)
	{
		list<Track *>::iterator to_rm, tmp;
		list<unsigned int>::iterator it;
		unsigned int index, removed;

		if (is_static())
			return;
		indices.sort();

		if (indices.back() >= plist.size())
			return;

		RENDER( modify_prepare() );

		index = 0;
		removed = 0;
		to_rm = plist.begin();
		for (it = indices.begin(); it != indices.end(); it++) {
			for (unsigned int i = 0; i < (*it - index); i++)
				to_rm++;
			remove_track_it(to_rm, (*it) - removed);
			index = *it;
			removed++;
		}

		schedule_save();
		RENDER( modify_done() );
	}

	void Playlist::add_sorted(list<Track *> &tracks)
	{
		list<Track *> copy;
		list<Track *>::iterator it, cur_it;
		unsigned int ins_index;

		tracks.sort(compare_tracks);
		copy = tracks;
		plist.merge(copy, compare_tracks);

		ins_index = 0;
		it = tracks.begin();
		for (cur_it = plist.begin(); cur_it != plist.end(); cur_it++) {
			if ((*cur_it) == (*it)) {
				add_track(*it, ins_index);
				if (it++ == tracks.end())
					break;
			};
			ins_index++;
		}
	}

	void Playlist::add_to_end(list<Track *> &tracks)
	{
		list<Track *>::iterator it;

		for (it = tracks.begin(); it != tracks.end(); it++) {
			plist.push_back(*it);
			add_track(*it, plist.size());
		};
	}

	void Playlist::add_to_front(list<Track *> &tracks)
	{
		list<Track *>::iterator it;

		tracks.reverse();
		for (it = tracks.begin(); it != tracks.end(); it++) {
			plist.push_front(*it);
			add_track(*it, 0);
		}
	}

	void Playlist::add_tracks(list<Track *> &tracks)
	{
		RENDER(modify_prepare());
		do_add_tracks(tracks);
		schedule_save();
		if (plist.size() == tracks.size())
			cur = plist.begin();
		RENDER(modify_done());
	}

}; /* namespace: libsaria */
