// Copyright (c) 2012 Bryan Schumaker.
#include <libsaria/playlist.h>
#include <libsaria/track.h>
#include <libsaria/print.h>
#include <libsaria/deck.h>
#include <libsaria/idle.h>

#include <stdlib.h>
#include <fstream>
#include <sstream>
using namespace std;

#define PLAYLIST_CURRENT_VERSION 2

static string plistdir = "playlist";

static string number_to_filename(unsigned int id)
{
	stringstream s;
	s << id;
	return s.str();
}

static void save_playlist(ofstream &stream, void *plist)
{
	libsaria::Playlist *playlist = (libsaria::Playlist *)plist;
	playlist->save(stream);
}

void read_plist(ifstream &stream)
{
	unsigned int version, type, size;
	unsigned int libid, trackid;
	bool disabled = false;
	list<libsaria::Track *> tracks;
	libsaria::Track *track;
	libsaria::Playlist *plist;

	stream >> version >> type >> size;
	if (version == 2)
		stream >> disabled;

	println("playlist type: %u size: %u disabled: %d", type, size, disabled);

	for (unsigned int i = 0; i < size; i++) {
		stream >> libid >> trackid;
		track = libsaria::library::lookup(libid, trackid);
		if (track)
			tracks.push_back(track);
	}

	if (tracks.size() > 0) {
		plist = libsaria::deck::new_playlist(tracks, (PlaylistType)type, false);
		plist->set_disabled(disabled);
	}
}

namespace libsaria
{

	void Playlist::save(ofstream &stream)
	{
		list<Track *>::iterator it;

		stream << PLAYLIST_CURRENT_VERSION << "\n";
		stream << type << " ";
		stream << plist.size() << " ";
		stream << is_disabled() <<"\n";

		for (it = plist.begin(); it != plist.end(); it++) {
			stream << (*it)->get_libid()	<< " ";
			stream << (*it)->get_id()	<< "\n";
		}
	}

	void Playlist::schedule_save()
	{
		string filepath;

		if (number < 0)
			return;
		else if (get_size() == 0) {
			rm_file();
			return;
		}

		filepath = plistdir + "/" + number_to_filename(number);
		app::save(filepath, save_playlist, &data_state, this);
	}

	void Playlist::rm_file()
	{
		string filepath;

		if (number < 0)
			return;

		idle::cancel_all(this);
		filepath = plistdir + "/" + number_to_filename(number);
		app::rm(filepath);
	}

	void deck::load_all()
	{
		app::read_numdir(plistdir, read_plist);
	}

}; /* Namespace: libsaria */
