// Copyright (c) 2012 Bryan Schumaker.
#include <libsaria/playlist.h>
#include <libsaria/renderer.h>
#include <libsaria/index.h>
#include <libsaria/prefs.h>
#include <libsaria/print.h>

#include <stdlib.h>

namespace libsaria
{

	void Playlist::incr_iter()
	{
		cur++;
		if (cur == plist.end())
			cur = plist.begin();
	}

	Track *Playlist::picked_next()
	{
		Track *res = (*cur);
		unsigned int index;

		index = find_cur_index();

		if (flags & PL_NO_DRAIN) {
			RENDER(goto_index(index));
			return res;
		}

		index = remove_track_it(cur, index);
		schedule_save();
		if (get_size() != 0)
			RENDER(goto_index(index));

		return res;
	}

	Track *Playlist::pick_sequential()
	{
		if (get_size() == 0)
			return NULL;

		if (flags & PL_NO_DRAIN)
			incr_iter();

		return picked_next();
	}

	Track *Playlist::pick_random()
	{
		unsigned int n, range = get_size();
		unsigned int min = 1;

		if (range == 0)
			return NULL;

		if (range == 1) {
			cur = plist.begin();
			return picked_next();
		}

		if (range > 10) {
			min   = (range / 10) + (range % 5);
			range = (range / 2);
		} else
			range *= 2;
		n = min + (rand() % range);
		//println("min: %u, max: %u, n: %u", min, min + range, n);

		for (unsigned int i = 0; i < n; i++)
			incr_iter();

		return picked_next();
	}

}; /* Namespace: libsaria */
