// Copyright (c) 2012 Bryan Schumaker.
#include <libsaria/index.h>
#include <libsaria/track.h>
#include <libsaria/library.h>
#include <libsaria/playlist.h>
#include <libsaria/renderer.h>

namespace libsaria
{

	Playlist::Playlist(string n, unsigned int f, PlaylistType t)
	{
		name = n;
		number = -1;
		flags = f;
		type = t;
		length = 0;
		renderer = NULL;
		cur = plist.begin();
	}

	Playlist::~Playlist()
	{
	}

	bool Playlist::is_static()
	{
		return (flags & PL_STATIC) == PL_STATIC;
	}

	bool Playlist::is_disabled()
	{
		return (flags & PL_DISABLED) == PL_DISABLED;
	}

	void Playlist::set_disabled(bool disabled)
	{
		if (disabled)
			flags |= PL_DISABLED;
		else
			flags &= ~PL_DISABLED;
		schedule_save();
		RENDER( on_disable(is_disabled()) );
	}

	void Playlist::track_updated(Track *track)
	{
		RENDER( track_updated(track) );
	}

	void Playlist::prepare_for_removal()
	{
		list<libsaria::Track *>::iterator it;

		/*
		 * I could do this in the destructor, but I think it could
		 * lead to double-free errors with static playlists
		 */
		if (renderer) {
			renderer->prepare_for_removal();
			if (!renderer->is_static()) {
				delete renderer;
				renderer = NULL;
			}
		}

		/* Remove playlist from tracks */
		for (it = plist.begin(); it != plist.end(); it++)
			(*it)->rm_playlist(this);

		rm_file();
	}

	void Playlist::set_filter_text(string &text)
	{
		index.do_filter(text);
		RENDER( refilter() );
	}

	bool Playlist::is_visible(libsaria::Track *track)
	{
		return index.is_visible(track);
	}

	string &Playlist::get_name()
	{
		return name;
	}

	void Playlist::renumber(int n)
	{
		if (number == n)
			return;
		if (n < number)
			rm_file();
		number = n;
		schedule_save();
		RENDER( renumbered(number) );
	}

	unsigned int Playlist::get_number()
	{
		return number;
	}

	unsigned int Playlist::get_size()
	{
		return plist.size();
	}

	unsigned int Playlist::get_length()
	{
		return length;
	}

	PlaylistType Playlist::get_type()
	{
		return type;
	}

	void Playlist::set_renderer(PlaylistRenderer *r)
	{
		renderer = r;
	}

	PlaylistRenderer *Playlist::get_renderer()
	{
		return renderer;
	}

	void Playlist::reset_iterator()
	{
		cur = plist.begin();
		RENDER(goto_index(0));
	}

	unsigned int Playlist::find_cur_index()
	{
		list<Track *>::iterator it;
		unsigned int index = 0;

		for (it = plist.begin(); it != cur; it++)
			index++;
		return index;
	}

}
