// Copyright (c) 2012 Bryan Schumaker
#include <ocarina/ocarina.h>
#include <ocarina/button.h>
#include <ocarina/body.h>

#include <libsaria/track.h>
#include <libsaria/deck.h>

static libsaria::Track *current = NULL;
static GtkWidget *title;
static GtkWidget *artist;
static GtkWidget *album;
static GtkWidget *ban;

static void set_label(GtkWidget *label, string text, string size)
{
	char *escaped = g_markup_escape_text(text.c_str(), -1);
	string markup = "<span size='" + size + "'>" + escaped + "</span>";
	gtk_label_set_markup(GTK_LABEL(label), markup.c_str());
	g_free(escaped);
}

static void refresh_widgets()
{
	string t = current->get_title();
	string ar = current->get_artist();
	string al = current->get_album();

	set_label(title, t, "xx-large");
	set_label(artist, ar, "x-large");
	set_label(album, al, "x-large");
}

static GtkWidget *align(GtkWidget *w)
{
	GtkWidget *alignment = gtk_alignment_new(0.0, 0.5, 0.0, 0.0);
	gtk_container_add(GTK_CONTAINER(alignment), w);
	return alignment;
}

static GtkWidget *make_tag_labels()
{
	GtkWidget *box = gtk_vbox_new(FALSE, 0);
	GtkWidget *window = gtk_scrolled_window_new(NULL, NULL);
	GtkWidget *viewport = gtk_viewport_new(NULL, NULL);

	title  = gtk_label_new("");
	artist = gtk_label_new("");
	album  = gtk_label_new("");

	gtk_scrolled_window_set_policy(GTK_SCROLLED_WINDOW(window),
				       GTK_POLICY_AUTOMATIC,
				       GTK_POLICY_AUTOMATIC);
	gtk_viewport_set_shadow_type(GTK_VIEWPORT(viewport), GTK_SHADOW_NONE);

	gtk_container_add(GTK_CONTAINER(viewport), box);
	gtk_container_add(GTK_CONTAINER(window), viewport);

	gtk_box_pack_start(GTK_BOX(box), align(title),  TRUE, FALSE, 0);
	gtk_box_pack_start(GTK_BOX(box), align(artist), TRUE, FALSE, 0);
	gtk_box_pack_start(GTK_BOX(box), align(album),  TRUE, FALSE, 0);

	return window;
}

static void on_ban(GtkWidget *w, gpointer d)
{
	bool banned = gtk_toggle_button_get_active(GTK_TOGGLE_BUTTON(ban));
	if (banned)
		change_button_image(ban, GTK_STOCK_NO);
	else
		change_button_image(ban, GTK_STOCK_YES);

	if (current->get_banned() == banned)
		return;

	current->set_banned(banned);
	if (banned)
		libsaria::deck::next();
}

static GtkWidget *make_buttons()
{
	GtkWidget *box = gtk_vbox_new(FALSE, 0);
	ban = make_toggle_button(GTK_STOCK_YES, on_ban, true);

	gtk_box_pack_start(GTK_BOX(box), ban, FALSE, FALSE, 0);

	return box;
}

namespace ocarina
{

	GtkWidget *body::now_playing_page()
	{
		GtkWidget *box = gtk_hbox_new(FALSE, 0);

		gtk_box_pack_start(GTK_BOX(box), make_tag_labels(), TRUE,  TRUE, 0);
		gtk_box_pack_start(GTK_BOX(box), make_buttons(),   FALSE, FALSE, 0);
		gtk_widget_show_all(box);

		return box;
	}

	void body::set_now_playing()
	{
		libsaria::Track *cur = libsaria::current_track();
		if (cur == current)
			return;
		current = cur;
		refresh_widgets();
		update_automatic_pause();
		gtk_toggle_button_set_active(GTK_TOGGLE_BUTTON(ban), current->get_banned());

		println("Current track changed");
	}

}; /* Namespace: ocarina */
