// Copyright (c) 2011 Bryan Schumaker
#include <ocarina/button.h>
#include <ocarina/chooser.h>

#include <libsaria/audio.h>
#include <libsaria/print.h>

static GtkWidget *get_button(void (* func)(GtkWidget *, gpointer),
			     gpointer data,
			     bool show)
{
	GtkWidget *button = gtk_button_new();
	g_signal_connect(button, "clicked", G_CALLBACK(func), data);
	if (show == true)
		gtk_widget_show(button);
	return button;
}

GtkWidget *get_image(const gchar *stockid, GtkIconSize size)
{
	GtkWidget *img = gtk_image_new_from_stock(stockid, size);
	gtk_widget_show(img);
	return img;
}

void change_button_image(GtkWidget *button, const gchar *stockid)
{
	GtkWidget *image = get_image(stockid, GTK_ICON_SIZE_MENU);
	gtk_button_set_image(GTK_BUTTON(button), image);
}

static GtkWidget *get_label(string text)
{
	GtkWidget *label = gtk_label_new(text.c_str());
	gtk_widget_show(label);
	return label;
}

GtkWidget *make_button_data(const gchar *stockid,
			    void (* func)(GtkWidget *, gpointer),
			    gpointer data,
			    bool show)
{
	GtkWidget *button = get_button(func, data, show);
	GtkWidget *image  = get_image(stockid, GTK_ICON_SIZE_MENU);
	gtk_button_set_image(GTK_BUTTON(button), image);
	return button;
}

GtkWidget *make_button(const gchar *stockid,
		       void (* func)(GtkWidget *, gpointer),
		       bool show)
{
	return make_button_data(stockid, func, NULL, show);
}


GtkWidget *make_larger_button(const gchar *stockid,
			      void (* func)(GtkWidget *, gpointer),
			      bool show)
{
	GtkWidget *button = get_button(func, NULL, show);
	GtkWidget *image  = get_image(stockid, GTK_ICON_SIZE_BUTTON);
	gtk_container_add(GTK_CONTAINER(button), image);
	return button;
}

GtkWidget *make_text_button(const gchar *stockid,
			    string text,
			    void (* func)(GtkWidget *, gpointer),
			    bool show)
{
	GtkWidget *button = get_button(func, NULL, show);
	GtkWidget *image  = get_image(stockid, GTK_ICON_SIZE_BUTTON);
	GtkWidget *label  = get_label(text);
	GtkWidget *box    = gtk_hbox_new(FALSE, 0);

	box_pack_start(box, image, FALSE, FALSE, 0);
	box_pack_start(box, label, FALSE, FALSE, 0);
	container_add(button, box);
	gtk_widget_show(box);
	return button;
}
