// Copyright (c) 2011 Bryan Schumaker
#include <version.h>
#include <libsaria/libsaria.h>
#include <libsaria/idle.h>

#include <ocarina/playlist.h>
#include <ocarina/ocarina.h>
#include <ocarina/window.h>
#include <ocarina/body.h>

static string lib_path;
static bool using_idle = false;

static void find_exe_path(string &res)
{
	char buf[1024];
	ssize_t len = readlink("/proc/self/exe", buf, sizeof(buf) -1);
	if (len != -1)
		res = string(buf);
}

static void rstrip_word(string &path)
{
	unsigned int index = path.rfind("/");
	if (index != string::npos)
		path = path.substr(0, index);
}

static void find_lib_path()
{
	string exe;

	find_exe_path(exe);
	if (exe == "")
		return;

	rstrip_word(exe); // ocarina-player
	rstrip_word(exe); // bin/

	lib_path = exe + "/lib/ocarina/";
	println("Lib directory? %s", lib_path.c_str());
}

static gboolean idle(gpointer data)
{
	libsaria::idle::run_task();
	using_idle = (libsaria::idle::size() != 0);
	if (using_idle)
		ocarina::body::set_idle_progress(libsaria::idle::progress());
	else
		ocarina::body::hide_idle_progress();
	return using_idle;
}

static void idle_add()
{
	if ((using_idle == false) && (libsaria::idle::size() > 0)) {
		println("Adding idle callback");
		g_idle_add(idle, NULL);
		using_idle = true;
	}
}

namespace ocarina
{

	string lib_file(string path)
	{
		string res = lib_path + "/" + path;
		println("Expanding to path: " + res);
		return res;
	}

	void quit()
	{
		gtk_main_quit();
	}

}

static gboolean timeout_poll(gpointer data)
{
	idle_add();
	ocarina::body::set_now_playing();
	ocarina::body::update_controls();
	return TRUE;
}

static void init(int argc, char **argv)
{
	gtk_init(&argc, &argv);
	ocarina::window::init();
	ocarina::body::init();
	ocarina::playlist::init();
	ocarina::init_pipe();
	gtk_timeout_add(100, timeout_poll, NULL);
}

int main(int argc, char **argv)
{
	struct libsaria::initdata ls_init = {
		argc,
		argv,
		"ocarina",	/* app name */
		true,		/* Use idle queue? */
		true,		/* Use global pipe? */
	};
	println("Ocarina " + vers_str());
	find_lib_path();

	libsaria::init(&ls_init);
	init(argc, argv);

	gtk_main();
	libsaria::quit();

	return 0;
}
