// Copyright (c) 2012 Bryan Schumaker
#include <ocarina/playlist.h>
#include <libsaria/track.h>

static string formatted(string str)
{
	char *escaped = g_markup_escape_text(str.c_str(), -1);
	string ret = escaped;
	g_free(escaped);
	return ret;
}

namespace ocarina
{

	void Playlist::modify_prepare()
	{
		gtk_widget_freeze_child_notify(treeview);
		gtk_tree_view_set_model(GTK_TREE_VIEW(treeview), NULL);
	}

	void Playlist::insert(libsaria::Track *track, unsigned int index)
	{
		gtk_list_store_insert_with_values(liststore, NULL, index,
			0, track,
			1, track->get_track(),
			2, track->get_title().c_str(),
			3, track->get_lenstr().c_str(),
			4, track->get_artist().c_str(),
			5, track->get_album().c_str(),
			6, track->get_year(),
			7, track->get_count(),
			8, track->get_last_played().c_str(),
			9, formatted(track->get_filepath()).c_str(),
			-1);
	}

	void Playlist::modify_done()
	{
		gtk_widget_thaw_child_notify(treeview);
		gtk_tree_view_set_model(GTK_TREE_VIEW(treeview), filter);
		set_label_text();
		if (playlist->get_size() == 0)
			gtk_widget_hide(box);
		else
			gtk_widget_show(box);
	}

	void Playlist::remove_index(unsigned int index)
	{
		GtkTreeIter iter;
		gtk_tree_model_iter_nth_child(GTK_TREE_MODEL(liststore), &iter, NULL, index);
		gtk_list_store_remove(liststore, &iter);
		set_label_text();
	}

	void Playlist::track_updated(libsaria::Track *track)
	{
		GtkTreeIter iter;
		libsaria::Track *t;

		gtk_tree_model_get_iter_first(GTK_TREE_MODEL(liststore), &iter);
		gtk_tree_model_get(GTK_TREE_MODEL(liststore), &iter, 0, &t, -1);

		while (t != track) {
			if (!gtk_tree_model_iter_next(GTK_TREE_MODEL(liststore), &iter))
				return;
			gtk_tree_model_get(GTK_TREE_MODEL(liststore), &iter, 0, &t, -1);
		}

		gtk_list_store_set(liststore, &iter,
			1, track->get_track(),
			2, track->get_title().c_str(),
			3, track->get_lenstr().c_str(),
			4, track->get_artist().c_str(),
			5, track->get_album().c_str(),
			6, track->get_year(),
			7, track->get_count(),
			8, track->get_last_played().c_str(),
			9, formatted(track->get_filepath()).c_str(),
			-1);
	}

};
