// Copyright (c) 2012 Bryan Schumaker
#include <ocarina/playlist.h>
#include "playlist.h"

#include <libsaria/track.h>
#include <libsaria/deck.h>

#include <list>
#include <sstream>
using namespace std;

#define BUTTON_RIGHT 3

struct PlaylistData {
	libsaria::Playlist *playlist;
	ocarina::Playlist *renderer;
};

static PlaylistData plist_data[MAX_PLAYLISTS];

static void new_playlist(ocarina::Playlist *plist, PlaylistType type, bool front)
{
	list<libsaria::Track *> tracks;

	plist->list_selected_tracks(tracks);
	if (tracks.size() == 0)
		return;

	plist->select_none();
	libsaria::deck::new_playlist(tracks, type, front);
	/*
	 * The new playlist will set the length string text,
	 * so reset it again here
	 */
	plist->on_show();
}

static void new_queue_front(GtkMenuItem *menu, gpointer data)
{
	new_playlist((ocarina::Playlist *)data, PLIST_QUEUE, true);
}

static void new_queue_back(GtkMenuItem *menu, gpointer data)
{
	new_playlist((ocarina::Playlist *)data, PLIST_QUEUE, false);
}

static void new_set_front(GtkMenuItem *menu, gpointer data)
{
	new_playlist((ocarina::Playlist *)data, PLIST_SET, true);
}

static void new_set_back(GtkMenuItem *menu, gpointer data)
{
	new_playlist((ocarina::Playlist *)data, PLIST_SET, false);
}

static void add_to_plist(GtkMenuItem *menu, gpointer data)
{
	list<libsaria::Track *> tracks;
	struct PlaylistData *plist = (struct PlaylistData *)data;

	plist->renderer->list_selected_tracks(tracks);
	if (tracks.size() == 0)
		return;

	plist->renderer->select_none();
	plist->playlist->add_tracks(tracks);

}

static GtkWidget *menu_item(const char *text, void (*func)(GtkMenuItem *, gpointer),
			    gpointer data)
{
	GtkWidget *widget = gtk_menu_item_new_with_label(text);
	g_signal_connect(widget, "activate", G_CALLBACK(func), data);
	return widget;
}

void show_rc_menu(GtkWidget *widget, GdkEvent *event, gpointer data)
{
	ocarina::Playlist *plist = (ocarina::Playlist *)data;

	if (event->button.button != BUTTON_RIGHT)
		return;
	plist->right_click(event->button.button, event->button.time);
}

static gboolean delete_banned(list<libsaria::Track *> &tracks, bool state)
{
	list<libsaria::Track *>::iterator it;

	if (tracks.size() == 0)
		return FALSE;

	for (it = tracks.begin(); it != tracks.end(); it++)
		(*it)->set_banned(state);
	return TRUE;
}

static gboolean delete_from_playlist(list<unsigned int> &indices, ocarina::Playlist *plist)
{
	if (indices.size() == 0)
		return FALSE;

	plist->get_playlist()->remove_indices(indices);
	libsaria::deck::garbage_collect();
	return TRUE;
}

gboolean key_pressed(GtkWidget *widget, GdkEvent *event, gpointer data)
{
	list<libsaria::Track *> tracks;
	list<unsigned int> indices;
	string key = gdk_keyval_name(event->key.keyval);
	libsaria::Playlist *list;
	ocarina::Playlist *plist = (ocarina::Playlist *)data;
	println("Treeview handling key press");

	if (key.substr(0, 3) == "KP_")
		key = key.substr(3);

	if (key == "s")
		new_playlist(plist, PLIST_SET, false);
	else if (key == "S")
		new_playlist(plist, PLIST_SET, true);
	else if (key == "q")
		new_playlist(plist, PLIST_QUEUE, false);
	else if (key == "Q")
		new_playlist(plist, PLIST_QUEUE, true);
	else if (key >= "0" && key <= "9") {
		unsigned int k;
		stringstream s(key);
		s >> k;
		plist->list_selected_tracks(tracks);
		if (tracks.size() == 0)
			return FALSE;
		list = libsaria::deck::get_playlist(k);
		if (list)
			list->add_tracks(tracks);
	} else if (key == "Delete") {
		string name = plist->get_playlist()->get_name();
		if (name == "Library" || name == "Banned") {
			plist->list_selected_tracks(tracks);
			return delete_banned(tracks, name == "Library");
		} else {
			plist->list_selected_indices(indices);
			return delete_from_playlist(indices, plist);
		}
	} else if (key == "Escape") {
		plist->select_none();
		return FALSE;
	} else
		return FALSE;
	return TRUE;
}

void make_playlist_items(GtkWidget *menu, ocarina::Playlist *renderer)
{
	GtkWidget *plist_menu, *sep;
	unsigned int i = 0;
	libsaria::Playlist *plist = libsaria::deck::get_playlist(i);

	while (plist != NULL) {
		stringstream s;
		s << "Add to " << plist->get_name() << " " << i;

		plist_data[i].playlist = plist;
		plist_data[i].renderer = renderer;

		plist_menu = menu_item(s.str().c_str(), add_to_plist, &plist_data[i]);
		gtk_menu_append(GTK_MENU(menu), plist_menu);
		i++;
		plist = libsaria::deck::get_playlist(i);
	}

	if (i != 0) {
		sep = gtk_separator_menu_item_new();
		gtk_menu_append(GTK_MENU(menu), sep);;
	}
}

namespace ocarina
{

	void Playlist::right_click(guint button, guint64 time)
	{
		GtkWidget *menu   = gtk_menu_new();
		GtkWidget *queue1 = menu_item("New queue (front)", new_queue_front, this);
		GtkWidget *set1   = menu_item("New set (front)",   new_set_front,   this);
		GtkWidget *sep    = gtk_separator_menu_item_new();
		GtkWidget *queue2 = menu_item("New queue (back)", new_queue_back, this);
		GtkWidget *set2   = menu_item("New set (back)",   new_set_back,   this);

		gtk_menu_append(GTK_MENU(menu), queue1);
		gtk_menu_append(GTK_MENU(menu), set1);
		gtk_menu_append(GTK_MENU(menu), sep);

		make_playlist_items(menu, this);

		gtk_menu_append(GTK_MENU(menu), queue2);
		gtk_menu_append(GTK_MENU(menu), set2);

		gtk_widget_show_all(menu);
		gtk_menu_popup(GTK_MENU(menu), NULL, NULL, NULL, NULL, button, time);
	}

}
