// Copyright (c) 2011 Bryan Schumaker

#include <version.h>
#include <ocarina/playlist.h>
#include <ocarina/ocarina.h>
#include <ocarina/window.h>

#include <libsaria/audio.h>
#include <libsaria/prefs.h>
#include <libsaria/deck.h>

#include <sstream>
using namespace std;

static GtkWidget *win;
static int old_w, old_h;
static bool ignore_configure_event = false;

static void resize(int w, int h)
{
	gtk_window_resize(GTK_WINDOW(win), w, h);
}

static void destroy(GtkWidget *widget, GdkEvent *event, gpointer data)
{
	ocarina::quit();
}

static gboolean configure(GtkWidget *widget, GdkEvent *event, gpointer data)
{
	int w, h;
	int old_w = libsaria::prefs::get("ocarina.window.w");
	int old_h = libsaria::prefs::get("ocarina.window.h");

	if (ignore_configure_event == true) {
		ignore_configure_event = false;
		return FALSE;
	}

	gtk_window_get_size(GTK_WINDOW(win), &w, &h);
	if ( (w == old_w) && (h == old_h) )
		return FALSE;

	if (w != old_w)
		libsaria::prefs::set("ocarina.window.w", w);
	if (h != old_h)
		libsaria::prefs::set("ocarina.window.h", h);

	println("New (w,h): (%d, %d)", w, h);
	gtk_window_resize(GTK_WINDOW(win), w, h);
	return FALSE;
}

static void window_state(GtkWidget *widget, GdkEvent *event, gpointer data)
{
	bool maxed = false;
	bool maximized = libsaria::prefs::get("ocarina.window.maximized");
	GdkWindowState state = gdk_window_get_state(GTK_WIDGET(win)->window);

	if (state & GDK_WINDOW_STATE_MAXIMIZED)
		maxed = true;

	if (maxed == maximized)
		return;

	println("Maximization changed to: %d", maxed);
	libsaria::prefs::set("ocarina.window.maximized", maxed);
	ignore_configure_event = true;
	if ( (maximized == false) && (maxed == true) ) {
		println("Restoring old (w,h): (%d, %d)", old_w, old_h);
		libsaria::prefs::set("ocarina.window.w", old_w);
		libsaria::prefs::set("ocarina.window.h", old_h);
	}
}

static gboolean key_pressed(GtkWidget *widget, GdkEvent *event, gpointer data)
{
	string key = gdk_keyval_name(event->key.keyval);
	GtkWidget *focus = gtk_window_get_focus(GTK_WINDOW(win));

	if (!(focus == NULL || focus == win))
		return FALSE;
	println("Window handling key press (1): " + key);

	if (key == "l" || key == "L")
		ocarina::playlist::switch_to_library();
	else if (key == "r" || key == "R")
		ocarina::playlist::switch_to_recent();
	else if (key == "b" || key == "B")
		ocarina::playlist::switch_to_banned();
	else if (key == "n")
		libsaria::deck::next();
	else if (key == "N")
		libsaria::deck::prev();
	else if (key == "space")
		libsaria::audio::toggle_play();
	else
		return FALSE;
	return TRUE;
}

static gboolean key_pressed_after(GtkWidget *widget, GdkEvent *event, gpointer data)
{
	string key = gdk_keyval_name(event->key.keyval);
	println("Window handling key press (2): " + key);

	if (key.substr(0, 3) == "KP_")
		key = key.substr(3);

	if (key >= "0" && key <= "9") {
		unsigned int k;
		stringstream s(key);
		s >> k;
		ocarina::playlist::switch_to_plist(k);
	} else if (key == "Escape")
		gtk_window_set_focus(GTK_WINDOW(win), NULL);
	else {
		println("Unhandled key: " + key);
		return FALSE;
	}
	return TRUE;
}

namespace ocarina
{

	void window::init()
	{
		win = gtk_window_new(GTK_WINDOW_TOPLEVEL);

		g_signal_connect(win, "destroy", G_CALLBACK(destroy), NULL);
		g_signal_connect(win, "configure-event", G_CALLBACK(configure), NULL);
		g_signal_connect(win, "window-state-event", G_CALLBACK(window_state), NULL);
		g_signal_connect(win, "key-press-event", G_CALLBACK(key_pressed), NULL);
		g_signal_connect_after(win, "key-press-event", G_CALLBACK(key_pressed_after), NULL);

		set_title("Ocarina " + vers_str());
		set_icon(lib_file("ocarina.png"));

		old_w = libsaria::prefs::init("ocarina.window.w", 800);
		old_h = libsaria::prefs::init("ocarina.window.h", 600);
		resize(old_w, old_h);

		if (libsaria::prefs::init("ocarina.window.maximized", false) == true)
			gtk_window_maximize(GTK_WINDOW(win));

		gtk_widget_show(win);
	}

	void window::set_title(string title)
	{
		gtk_window_set_title(GTK_WINDOW(win), title.c_str());
	}

	void window::set_icon(string path)
	{
		gtk_window_set_icon_from_file(GTK_WINDOW(win), path.c_str(), NULL);
	}

	void window::set_content(GtkWidget *content)
	{
		gtk_container_add(GTK_CONTAINER(win), content);
	}

} /* Namespace: ocarina */
